import java.io.*;

public class Project {
	public ListModule mods = new ListModule();
	public ListLink links = new ListLink();
	public String modName = "R4 Test Scene";
	public String modAuthor = "Gordon Williams";

	public void loadFile(String fn) throws Exception {
		BufferedReader F = new BufferedReader(new FileReader(fn));
		String L = F.readLine();
		while (L!=null) {
			if (L.equals("MODULE:")) mods.load(F);
			if (L.equals("LINK:")) links.load(F);
			L = F.readLine();
		}
		F.close();
	}

	public void saveFile(String fn) throws Exception {
		BufferedWriter F = new BufferedWriter(new FileWriter(fn));
		mods.save(F);
		links.save(F);
		F.close();
	}

	public void buildR4(String fn) throws Exception {
		BufferedWriter F = new BufferedWriter(new FileWriter(fn));
		F.write("SCENE (\n");
		F.write(" \"name\" = \""+modName+"\";\n");
		F.write(" \"author\" = \""+modAuthor+"\";\n");
		// order the modules
		Module M[] = Const.proj.mods.orderModules();
		// calculate module names
		int x,y,c=0;
		// module names
		for (x=0;x<M.length;x++) {
			int mc = M[x].getPropI("MODULE_C",0);
			M[x].idModule = new String[mc];
			for (y=0;y<mc;y++) {
				M[x].idModule[y] = "MOD"+(c++);
			}
		}
		// variable names
		c=0;
		for (x=0;x<M.length;x++) {
			int vc = M[x].getPropI("VAR_C",0);
			M[x].idVar = new String[vc];
			for (y=0;y<vc;y++) {
				M[x].idVar[y] = "V"+(c++);
			}
		}
		// module input names
		for (x=0;x<M.length;x++) {
			int ic = M[x].getPropI("INPUT_C",0);
			M[x].idInput = new String[ic];
			for (y=0;y<ic;y++) {
				Module md = Const.proj.links.getLink(M[x],y);
				if (md==null) throw new Exception("No input to module ["+md.NAME+"]");
				M[x].idInput[y] = md.idModule[md.idModule.length-1];
			}
		}
		// output the module part
		for (y=M.length-1;y>=0;y--) {
			F.write(" // "+M[y].NAME+"\n");
			int sc = M[y].getPropI("SEC_MODULE_C",0);
			for (x=1;x<=sc;x++) {
				F.write(" "+M[y].replaceText("SEC_MODULE_"+x)+"\n");
			}

		}
		F.write(")\n\n");
		for (y=M.length-1;y>=0;y--) {
			int sc = M[y].getPropI("SEC_VAR_C",0);
			if (sc>0) F.write("// "+M[y].NAME+"\n");
			for (x=1;x<=sc;x++) {
				F.write(M[y].replaceText("SEC_VAR_"+x)+"\n");
			}
		}
		F.write("\n\n");
		F.write("void init() {\n");
		for (y=M.length-1;y>=0;y--) {
			int sc = M[y].getPropI("SEC_INIT_C",0);
			if (sc>0) F.write(" // "+M[y].NAME+"\n");
			for (x=1;x<=sc;x++) {
				F.write(" "+M[y].replaceText("SEC_INIT_"+x)+"\n");
			}
		}
		F.write("}\n\n");
		F.write("void reset() {\n");
		for (y=M.length-1;y>=0;y--) {
			int sc = M[y].getPropI("SEC_RESET_C",0);
			if (sc>0) F.write(" // "+M[y].NAME+"\n");
			for (x=1;x<=sc;x++) {
				F.write(" "+M[y].replaceText("SEC_RESET_"+x)+"\n");
			}
		}
		F.write("}\n\n");
		F.write("void render() {\n");
		for (y=M.length-1;y>=0;y--) {
			int sc = M[y].getPropI("SEC_RENDER_C",0);
			if (sc>0) F.write(" // "+M[y].NAME+"\n");
			for (x=1;x<=sc;x++) {
				F.write(" "+M[y].replaceText("SEC_RENDER_"+x)+"\n");
			}
		}
		F.write("}\n\n");
		F.close();
	}

	public void onExit() {
		try {
			System.out.println("Saving...");
			saveFile("backup.r4src");
			System.out.println("R4ing...");
			buildR4("result.r4");
		} catch (Exception ex) { ex.printStackTrace(); }
	}
}