import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class WinDesign extends JFrame implements WindowListener,MouseListener,MouseMotionListener {

	String status = "";
	Module selected = null;
	int selectedIO = -2;

	public final static int MODE_MOVE = 0;
	public final static int MODE_ADD  = 1;
	public final static int MODE_REMOVE = 2;
	public final static int MODE_JOIN = 3;
	private int mode = -1;

	public void setMode(int _mode) {
		mode = _mode;
		selected = null;
		selectedIO = -2;
		if (mode==MODE_MOVE) setStatus("Click and Drag to move modules...");
		if (mode==MODE_ADD) setStatus("Click to add a module...");
		if (mode==MODE_REMOVE) setStatus("Click to remove a module...");
		if (mode==MODE_JOIN) setStatus("Drag between two red boxes to join modules...");
	}

	public WinDesign() {
		super("Design");
		setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		setSize(512,512);
		// add listeners
		getContentPane().addMouseMotionListener(this);
		getContentPane().addMouseListener(this);
		addWindowListener(this);
		setMode(MODE_ADD);
	}

	public void paint(Graphics g) {
		if (g==null) return;
		super.paint(g);

		Graphics gr = getContentPane().getGraphics();

		Dimension D = getContentPane().getSize();
		//clear
		gr.setColor(Color.WHITE);
		gr.fillRect(0,0,D.width,D.height);
		// draw
		Const.proj.mods.draw(gr);
		Const.proj.links.draw(gr);
		// status bar
		gr.setColor(Color.GRAY);
		gr.fillRect(0,D.height-15,D.width,15);
		gr.setColor(Color.BLACK);
		gr.drawString(status,4,D.height-2);
	}

	public void setStatus(String _status) {
		status = _status;
		update();
	}

	public void update() {
		paint(getGraphics());
	}

	// Listeners
	public void mouseDragged(MouseEvent e) {
		if (mode==MODE_MOVE) {
			if (selected!=null) {
				selected.pos.x = e.getX();
			    selected.pos.y = e.getY();
				update();
			}
		}
	}
	public void mouseMoved(MouseEvent e) {}
	public void mouseClicked(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {
		if (mode==MODE_ADD) {
			FileDialog FD = new FileDialog(this,"Open Module",FileDialog.LOAD);
			FD.setDirectory(Const.MODULEDIR);
			//FD.setFilenameFilter
			FD.show();

			if (FD.getFile()!=null) {
				try {
					Module M = new Module();
					M.loadFile(FD.getFile());
					Const.proj.mods.add(M);
					M.pos = new Pt(e.getX(),e.getY());
				} catch (Exception ex) { ex.printStackTrace(); }			
			}
			update();
		}
		if (mode==MODE_MOVE) {
			selected = Const.proj.mods.findModule(new Pt(e.getX(),e.getY()));
			Const.winFunc.setModule(selected);	// set function window up
		}
		if (mode==MODE_JOIN) {
			selected = Const.proj.mods.findModuleIO(new Pt(e.getX(),e.getY()));
			if (selected!=null) selectedIO = selected.IOnum;
		}
		if (mode==MODE_REMOVE) {
			selected = Const.proj.mods.findModule(new Pt(e.getX(),e.getY()));
			if (selected!=null) {
				Const.proj.mods.remove(selected);
				for (int i=-1;i<selected.inputcount;i++)
					Const.proj.links.remove(selected,i);
				update();
			} else {
				selected = Const.proj.mods.findModuleIO(new Pt(e.getX(),e.getY()));
				if (selected!=null) {
					Const.proj.links.remove(selected,selected.IOnum);
					update();
				}
			}
		}
	}
	public void mouseReleased(MouseEvent e) {
		if (mode==MODE_MOVE)
			selected = null;
		if (mode==MODE_JOIN) {
			Module M = Const.proj.mods.findModuleIO(new Pt(e.getX(),e.getY()));
			if ((M!=null) && (selected!=null) && (M!=selected)) {
				if ((M.IOnum>=0) && (selectedIO==-1) && (!Const.proj.links.linkTo(M,M.IOnum))) {
					Const.proj.links.add(selected,M,M.IOnum);
					update();
				}
				if ((M.IOnum==-1) && (selectedIO>=0) && (!Const.proj.links.linkTo(selected,selectedIO))) {
					Const.proj.links.add(M,selected,selectedIO);
					update();
				}
			}
		}
	}

	// -----------------------------------------------------------
	public void windowActivated(WindowEvent e) {}
	public void windowClosed(WindowEvent e) {}
	public void windowClosing(WindowEvent e) {
		Const.proj.onExit();
	}
	public void windowDeactivated(WindowEvent e) {}
	public void windowDeiconified(WindowEvent e) {}
	public void windowIconified(WindowEvent e) {} 
	public void windowOpened(WindowEvent e) {}

}